﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using CoreAudioApi;


namespace Audioslave {
    public partial class dummy : Form {

        enum TrayIconType {
            None,
            Speaker,
            SpeakerHearing,
            SpeakerOff,
            SpeakerWarn
        }

        private NotifyIcon trayicon = new NotifyIcon();
        private Timer tmr_mutedelay = new Timer();
        private Timer tmr_automute = new Timer();
        int automute_timeout;
        MMDevice audioDevice;
        bool hwfound = true;



        public dummy() {
            this.ShowInTaskbar = false;
            this.Visible = false;
            InitializeComponent();

            

            this.Icon = Icon.FromHandle(Audioslave.Properties.Resources.speaker.GetHicon());
            SetIcon(TrayIconType.Speaker);
            this.trayicon.ContextMenu = CreateMenu();
            this.trayicon.Visible = true;

            MMDeviceEnumerator DevEnum = new MMDeviceEnumerator();
            audioDevice = DevEnum.GetDefaultAudioEndpoint(EDataFlow.eRender, ERole.eMultimedia);
            audioDevice.AudioEndpointVolume.OnVolumeNotification += new AudioEndpointVolumeNotificationDelegate(AudioEndpointVolume_OnVolumeNotification);            

            tmr_mutedelay.Tick += new EventHandler(tmr_mutedelay_Tick);
            this.tmr_mutedelay.Interval = 5000;
            this.tmr_mutedelay.Enabled = false;

            tmr_automute.Tick += new EventHandler(tmr_automute_Tick);
            tmr_automute.Interval = 2000;
            if(Properties.Settings.Default.reactOnsound == true) {
                tmr_automute.Enabled = true;
                automute_timeout = Properties.Settings.Default.delayAutoMute;
            }
        
        }

        void tmr_automute_Tick(object sender, EventArgs e) {
            if(audioDevice.AudioEndpointVolume.Mute == true) return; //Bei Mute ignorieren
            if(audioDevice.AudioMeterInformation.MasterPeakValue > Properties.Settings.Default.automuteThreshold) {
                this.automute_timeout = Properties.Settings.Default.delayAutoMute;
                SetIcon(TrayIconType.SpeakerHearing);
            } else {
                SetIcon(TrayIconType.Speaker);
                this.automute_timeout -= (tmr_automute.Interval / 1000);
                if(this.automute_timeout <= 0) {
                    this.audioDevice.AudioEndpointVolume.Mute = true;
                    SetServo(false);
                    this.tmr_automute.Enabled = false;
                }
            }
            System.Diagnostics.Debug.WriteLine(this.automute_timeout);
        }


        MenuItem mi_reactOnMute;
        MenuItem mi_reactOnSilence;
        USBservo servo;

        private ContextMenu CreateMenu() {
            ContextMenu mnu = new ContextMenu();


            MenuItem mi = new MenuItem("Einstellungen");
            mi.Click += new EventHandler(miConfig_Click);
            mnu.MenuItems.Add(mi);

            mnu.MenuItems.Add("-");

            this.mi_reactOnSilence = new MenuItem("Auf Stille reagieren");
            this.mi_reactOnSilence.Click += new EventHandler(miReactOnsound_Click);
            this.mi_reactOnSilence.Checked = Properties.Settings.Default.reactOnsound;
            mnu.MenuItems.Add(this.mi_reactOnSilence);

            this.mi_reactOnMute = new MenuItem("Auf Mute reagieren");
            this.mi_reactOnMute.Click += new EventHandler(miReactOnmute_Click);
            this.mi_reactOnMute.Checked = Properties.Settings.Default.reactOnmute;
            mnu.MenuItems.Add(this.mi_reactOnMute);

            mnu.MenuItems.Add("-");

            mi = new MenuItem("Anlage einschalten");
            mi.Click += new EventHandler(miSwitchOn_Click);
            mnu.MenuItems.Add(mi);

            mi = new MenuItem("Anlage ausschalten");
            mi.Click += new EventHandler(miSwitchOff_Click);
            mnu.MenuItems.Add(mi);

            mnu.MenuItems.Add("-");

            mi = new MenuItem("Beenden");
            mi.Click += new EventHandler(miEnd_Click);
            mnu.MenuItems.Add(mi);

            return mnu;

        }

        TrayIconType lasticontype = TrayIconType.None;
        private void SetIcon(TrayIconType icontype) {
            Bitmap bmp;

            if(hwfound == false) {
                icontype = TrayIconType.SpeakerWarn;
            }

            if(lasticontype == icontype) {
                return;
            }

            

            switch(icontype) {
                case TrayIconType.SpeakerHearing:
                    bmp = Properties.Resources.speaker_hearing;
                    this.trayicon.Text = "Warte auf Stille...";
                    break;
                case TrayIconType.SpeakerOff:
                    bmp = Properties.Resources.speaker_off;
                    this.trayicon.Text = "Verstärker abgeschaltet";
                    break;
                case TrayIconType.SpeakerWarn:
                    bmp = Properties.Resources.speaker_warn;
                    this.trayicon.Text = "Hardware nicht gefunden!";
                    break;
                default:
                case TrayIconType.Speaker:
                    bmp = Properties.Resources.speaker;
                    this.trayicon.Text = "";
                    break;
            }
            this.trayicon.Icon = Icon.FromHandle(bmp.GetHicon());

        }

        void miConfig_Click(object sender, EventArgs e) {
            frmConfig frm = new frmConfig();
            if(frm.ShowDialog() == DialogResult.Yes) {
                //Daten wurden gespeichert/geändert
                //TODO
                MessageBox.Show("Bitte Anwendung neustarten, war bis jetzt zu faul, das Feature zu programmieren.");
            }
            frm.Dispose();
            

        }

        void miReactOnmute_Click(object sender, EventArgs e) {
            MenuItem s = (MenuItem)sender;
            Properties.Settings.Default.reactOnmute = !Properties.Settings.Default.reactOnmute;
            Properties.Settings.Default.Save();
            s.Checked = Properties.Settings.Default.reactOnmute;
        }

        void miReactOnsound_Click(object sender, EventArgs e) {
            MenuItem s = (MenuItem)sender;
            Properties.Settings.Default.reactOnsound = !Properties.Settings.Default.reactOnsound;
            Properties.Settings.Default.Save();
            s.Checked = Properties.Settings.Default.reactOnsound;
            this.tmr_automute.Enabled = Properties.Settings.Default.reactOnsound;
            this.automute_timeout = Properties.Settings.Default.delayAutoMute;
        }



        
        private void tmr_mutedelay_Tick(object sender, EventArgs e) {
            this.tmr_mutedelay.Enabled = false;
            SetServo(false);
            
        }


        private void SetServo(bool power) {
            if(this.servo == null) {
                this.servo = new USBservo(Properties.Settings.Default.usbDevice);
            }
            hwfound = true;

            if(power == true) {
                SetIcon(TrayIconType.Speaker);
                if(Properties.Settings.Default.reactOnsound == true) {
                    this.automute_timeout = Properties.Settings.Default.delayAutoMute;
                    this.tmr_automute.Enabled = true;
                }
            } else {
                SetIcon(TrayIconType.SpeakerOff);
                tmr_automute.Enabled = false; //Wozu Rechenzeit verschwenden
            }

            byte value = (byte)(power == true ? Properties.Settings.Default.servoposOn : Properties.Settings.Default.servoposOff);

            if(this.servo.SetAngle((byte)value, (byte)Properties.Settings.Default.servoDuration) == false) {
                //Servo nicht mehr erreichbar, nochmal versuchen
                this.servo.Close();
                this.servo = new USBservo(Properties.Settings.Default.usbDevice);
                if(this.servo.SetAngle((byte)value, (byte)Properties.Settings.Default.servoDuration) == false) {
                    SetIcon(TrayIconType.SpeakerWarn);
                    hwfound = false;
                }
            }
        }


        bool lastmute = false;

        private void AudioEndpointVolume_OnVolumeNotification(AudioVolumeNotificationData data) {
            if(this.InvokeRequired) {
                this.Invoke(new AudioEndpointVolumeNotificationDelegate(AudioEndpointVolume_OnVolumeNotification), new object[] { data });
            } else {
                if(lastmute != data.Muted) {
                    if(data.Muted) {
                        tmr_mutedelay.Enabled = true;
                        
                    } else {
                        tmr_mutedelay.Enabled = false;
                        SetServo(true);
                    }
                }
                lastmute = data.Muted;
            }
        }

        void miSwitchOff_Click(object sender, EventArgs e) {
            SetServo(false);
        }

        void miSwitchOn_Click(object sender, EventArgs e) {
            SetServo(true);
        }

        void miEnd_Click(object sender, EventArgs e) {
            if(MessageBox.Show("Möchten Sie die Anwendung wirklich beenden?", this.Text, MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) == DialogResult.Yes) {
                this.trayicon.Dispose();
                Application.Exit();
            }
            
        }
    }
}
