﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using CoreAudioApi;
using System.IO.USB;

namespace Audioslave {

    public partial class frmConfig : Form {
        public frmConfig() {
            InitializeComponent();
        }

        MMDevice audiodevice = null;

        private void frmConfig_Load(object sender, EventArgs e) {
            Timer tmr_audio = new Timer();
            tmr_audio.Interval = 20;
            tmr_audio.Tick += new EventHandler(tmr_audio_Tick);
            tmr_audio.Enabled = true;

            BuildAudioList();
            BuildUSBList();
            chkReactonSilence.Checked = Properties.Settings.Default.reactOnsound;
            chkReactonMute.Checked = Properties.Settings.Default.reactOnmute;
            numSilenceThreshold.Value = (int)(Properties.Settings.Default.automuteThreshold * 100);
            numSilenceDelay.Value = Properties.Settings.Default.delayAutoMute;

            numMuteDelay.Value = Properties.Settings.Default.delayMute;
            numServoOff.Value = Properties.Settings.Default.servoposOff;
            numServoOn.Value = Properties.Settings.Default.servoposOn;
            numServoDuration.Value = Properties.Settings.Default.servoDuration * 10;
            


        }

        void tmr_audio_Tick(object sender, EventArgs e) {
            if(this.audiodevice != null) {
                int volume = (int)(this.audiodevice.AudioMeterInformation.MasterPeakValue * 100);
                this.prgVolume.Value = volume;
                this.lblVolume.Text = volume.ToString() + "%";
                
            }
        }

        private void BuildAudioList() {
            cboAudioDevice.Items.Clear();
            MMDeviceEnumerator DevEnum = new MMDeviceEnumerator();
            MMDevice defaultdevice = DevEnum.GetDefaultAudioEndpoint(EDataFlow.eRender, ERole.eMultimedia);

            MMDeviceCollection devcoll = DevEnum.EnumerateAudioEndPoints(EDataFlow.eRender, EDeviceState.DEVICE_STATEMASK_ALL);
            for(int i = 0; i < devcoll.Count; i++) {
                AudioDeviceListItem item = new AudioDeviceListItem { Text = devcoll[i].FriendlyName, ID = devcoll[i].ID };
                if(devcoll[i].ID.CompareTo(defaultdevice.ID) == 0) {
                    item.Text += " **Standardgerät**";
                }
                int idx = cboAudioDevice.Items.Add(item);

                if(item.ID.CompareTo(Audioslave.Properties.Settings.Default.audioDevice) == 0) {
                    cboAudioDevice.SelectedIndex = idx;
                }
            }
        }


        private void BuildUSBList() {
            cboUSBDevice.Items.Clear();
            List<EnumHidDevice> hiddevices = HIDImports.EnumerateDevices();

            for(int i = 0; i < hiddevices.Count; i++) {
                EnumHidDevice dev = hiddevices[i];
                if(dev.VendorID == 0 || dev.ProductID == 0) continue;
                HidDeviceListItem item = new HidDeviceListItem { Device = dev };
                int idx = cboUSBDevice.Items.Add(item);

                if(dev.DevicePath.CompareTo(Audioslave.Properties.Settings.Default.usbDevice) == 0) {
                    cboUSBDevice.SelectedIndex = idx;
                }
            }
        }

        private void cboAudioDevice_SelectedIndexChanged(object sender, EventArgs e) {
            AudioDeviceListItem item = (AudioDeviceListItem)(((ComboBox)sender).SelectedItem);
            MMDeviceEnumerator DevEnum = new MMDeviceEnumerator();
            this.audiodevice = DevEnum.GetDevice(item.ID);
        }

        private void btnSave_Click(object sender, EventArgs e) {
            Properties.Settings.Default.audioDevice = ((AudioDeviceListItem)cboAudioDevice.SelectedItem).ID;
            Properties.Settings.Default.usbDevice = ((HidDeviceListItem)cboUSBDevice.SelectedItem).Device.DevicePath;
            
            Properties.Settings.Default.reactOnsound = chkReactonSilence.Checked;
            Properties.Settings.Default.reactOnmute = chkReactonMute.Checked;
            Properties.Settings.Default.automuteThreshold = (float)numSilenceThreshold.Value / 100;
            Properties.Settings.Default.delayAutoMute = (int)numSilenceDelay.Value;

            Properties.Settings.Default.delayMute = (int)numMuteDelay.Value;
            Properties.Settings.Default.servoposOff = (int)numServoOff.Value;
            Properties.Settings.Default.servoposOn = (int)numServoOn.Value;
            Properties.Settings.Default.servoDuration = (int)(numServoDuration.Value / 10);
            Properties.Settings.Default.Save();

            this.DialogResult = DialogResult.Yes;
            this.Close();
        }

        private void cmdAbort_Click(object sender, EventArgs e) {
            this.DialogResult = DialogResult.Abort;
            this.Close();
        }

        private void btnSetOffpos_Click(object sender, EventArgs e) {
            HidDeviceListItem hiditem = (HidDeviceListItem)cboUSBDevice.SelectedItem;
            if(hiditem == null) {
                MessageBox.Show("Es wurde kein USB-Gerät ausgewählt!", this.Text, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
            USBservo servo = new USBservo(hiditem.Device.DevicePath);
            if(servo.SetAngle((byte)numServoOff.Value, (byte)(numServoDuration.Value / 10)) == false) {
                MessageBox.Show("Position konnte nicht gesetzt werden.\nHardware nicht angeschlossen?", this.Text, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
            servo.Close();
        }

        private void btnSetOnpos_Click(object sender, EventArgs e) {
            HidDeviceListItem hiditem = (HidDeviceListItem)cboUSBDevice.SelectedItem;
            if(hiditem == null) {
                MessageBox.Show("Es wurde kein USB-Gerät ausgewählt!", this.Text, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
            USBservo servo = new USBservo(hiditem.Device.DevicePath);
            if(servo.SetAngle((byte)numServoOn.Value, (byte)(numServoDuration.Value / 10)) == false) {
                MessageBox.Show("Position konnte nicht gesetzt werden.\nHardware nicht angeschlossen?", this.Text, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
            servo.Close();

        }
    }
}
