﻿using System;
using System.Runtime.InteropServices;
using System.Diagnostics;
using System.IO;
using System.Text;
using System.Runtime.Serialization;
using Microsoft.Win32.SafeHandles;
using System.Threading;
using System.IO.USB;

namespace Audioslave {
    public class USBservo {

        private SafeFileHandle mHandle;

        private byte reportCount;

        /// <summary>
        /// Erstellt ein neues servo-Objekt mit Anbindung an die Hardware
        /// </summary>
        /// <param name="VendorID">USB-Vendor-ID der Hardware</param>
        /// <param name="ProductID">USB-Product-ID der Hardware</param>
        public USBservo(int VendorID, int ProductID) {

            
            this.reportCount = (byte)(2 + 1);

            int index = 0;
            bool found = false;
            Guid guid;

            HIDImports.HidD_GetHidGuid(out guid);

            IntPtr hDevInfo = HIDImports.SetupDiGetClassDevs(ref guid, null, IntPtr.Zero, HIDImports.DIGCF_DEVICEINTERFACE);

            HIDImports.SP_DEVICE_INTERFACE_DATA diData = new HIDImports.SP_DEVICE_INTERFACE_DATA();
            diData.cbSize = Marshal.SizeOf(diData);

            while(HIDImports.SetupDiEnumDeviceInterfaces(hDevInfo, IntPtr.Zero, ref guid, index, ref diData)) {
                UInt32 size;

                HIDImports.SetupDiGetDeviceInterfaceDetail(hDevInfo, ref diData, IntPtr.Zero, 0, out size, IntPtr.Zero);

                HIDImports.SP_DEVICE_INTERFACE_DETAIL_DATA diDetail = new HIDImports.SP_DEVICE_INTERFACE_DETAIL_DATA();
                diDetail.cbSize = (uint)(IntPtr.Size == 8 ? 8 : 5);

                if(HIDImports.SetupDiGetDeviceInterfaceDetail(hDevInfo, ref diData, ref diDetail, size, out size, IntPtr.Zero)) {
                    Debug.WriteLine(index + " " + diDetail.DevicePath + " " + Marshal.GetLastWin32Error());
                    

                    mHandle = HIDImports.CreateFile(diDetail.DevicePath, FileAccess.ReadWrite, FileShare.ReadWrite, IntPtr.Zero, FileMode.Open, HIDImports.EFileAttributes.Overlapped, IntPtr.Zero);
                    string manufacturer = HIDImports.GetManufacturerString(mHandle.DangerousGetHandle());
                    string device = HIDImports.GetProductString(mHandle.DangerousGetHandle());
                    string serialnumber = HIDImports.GetSerialNumberString(mHandle.DangerousGetHandle());
                    
                    Debug.WriteLine(String.Format("{0} {1} {2}", manufacturer, device, serialnumber));

                    HIDImports.HIDD_ATTRIBUTES attrib = new HIDImports.HIDD_ATTRIBUTES();
                    attrib.Size = Marshal.SizeOf(attrib);

                    if(HIDImports.HidD_GetAttributes(mHandle.DangerousGetHandle(), ref attrib)) {

                        if(attrib.VendorID == VendorID && attrib.ProductID == ProductID) {
                            found = true;
                            break;
                        } else {
                            mHandle.Close();
                        }
                    }
                } else {
                    throw new Exception("SetupDiGetDeviceInterfaceDetail failed on index " + index);
                }
                index++;
            }
            HIDImports.SetupDiDestroyDeviceInfoList(hDevInfo);

            if(found == false) throw new Exception("Device not found");
        }

        public USBservo(string DevicePath) {
            this.mHandle = HIDImports.GetDeviceByPath(DevicePath);
            this.reportCount = 3;
        }

        ~USBservo() {
            if(this.mHandle != null)
                this.mHandle.Close();
        }

        public bool SetAngle(byte angle, byte duration) {
            if(this.mHandle.IsInvalid == true) { return false; }

            byte[] outdata = new byte[this.reportCount];

            outdata[0] = 0;
            outdata[1] = angle;
            outdata[2] = duration;

            IntPtr handle = this.mHandle.DangerousGetHandle();
            return HIDImports.HidD_SetFeature(handle, outdata, (uint)outdata.Length);
        }


        public void Close() {
            HIDImports.CloseHandle(this.mHandle.DangerousGetHandle());
        }
    }
}
